/* ui_defs.h
   @(#)ui_defs.h	1.1 ARGOS 93/12/13 17:30:33

   Definitions of structures for user interface.
   Used (in part) by both the uitoc preprocessor,
   and the gui runtime program. */

#ifndef UI_DEFS
#define UI_DEFS

#include <stdio.h>

#define UIMaxDepth 10

#define UIMaxNodes 2000

#define UP_CODE       1
#define DOWN_CODE    -1

/* Kinds of syntax node. */

#define AndNode      1
#define OrNode       2
#define ImmNode      3
#define TermNode     4
#define SwitchNode   5
#define ForNode      6
#define LayoutNode   7
#define LabelNode    8

/* Kinds of terminal (user input) */

#define BOOLEAN_TERM 1
#define CHOICE_TERM  2
#define STRING_TERM  3
#define INT_TERM     4
#define FLOAT_TERM   5
#define DOUBLE_TERM  6
#define CYCLE_TERM   7
#define STRUCT_TERM  8
#define SELECT_TERM  9

/* Layout styles (zero is default) */

#define BOX_LAYOUT  0
#define BAR_LAYOUT  1
#define DROP_LAYOUT 2

typedef int NODEID;

typedef struct UINODE {
  char *name;
  char *help;
  char *on_cr;

  int level;
  int kind;
  int term_kind;

/* indices of other related nodes */

  int child;
  int sibling;
  int back;

/* index of child that is the popup menu if any */

  int popup;

  int layout_style;

/* values to be set during run time */

  long struct_ptr;
  long flags;

/* like it says */

  int spare;

#ifdef UITOC_COMM

/* translation-time,
   mostly function block indices */

  int follows;
  int overlays;

  int layout_left;
  int layout_right;
  int layout_tab;
  int layout_vtab;

  int place_x;
  int place_y;

  int window_x;
  int window_y;
  int window_w;
  int window_h;

  char *struc_size;

  int enable_choice;
  int omit;
  int click_action;
  int auto_ok;
  int read_only;
  int enable_ok;

  int remain;
  int confirm;
  int conf_msg;
  int discard;
  int event_test;
  int no_label;

  int fill_color;

  int body_color;
  int on_color;
  int label_color;
  int mouse_color;

  int summary;
  int def_pick;
  int auto_select;
  int highlight;

  int on_entry;
  int on_exit;
  int action;

  int for_list;

  int pick;
  int no_pick;

  int rfile;

  int choice_set;
  int choice_init;
  int choice_allow;
  int choice_disallow;
  int choice_min;
  int choice_max;

  int choice_key_case;

  int str_no_echo;

  char *term_encode;
  char *term_decode;

  int term_width;

  int long_init;
  int long_min;
  int long_max;

  int float_init;
  int float_min;
  int float_max;

  int double_init;
  int double_min;
  int double_max;

#else

/* run-time,
   pointers to functions */

  char *(*follows)(NODEID);
  char *(*overlays)(NODEID);

  long (*layout_left)(NODEID);
  long (*layout_right)(NODEID);
  long (*layout_tab)(NODEID);
  long (*layout_vtab)(NODEID);

  long (*place_x)(NODEID);
  long (*place_y)(NODEID);

  long (*window_x)(NODEID);
  long (*window_y)(NODEID);
  long (*window_w)(NODEID);
  long (*window_h)(NODEID);

  int  struc_size;

  long (*enable_choice)(NODEID);
  long (*omit)(NODEID);
  long (*click_action)(NODEID);
  long (*auto_ok)(NODEID);
  long (*read_only)(NODEID);
  long (*enable_ok)(NODEID);

  long (*remain)(NODEID);
  long (*confirm)(NODEID);
  char *(*conf_msg)(NODEID);
  long (*discard)(NODEID);
  long (*event_test)(NODEID);
  long (*no_label)(NODEID);

  long (*fill_color)(NODEID);

  long (*body_color)(NODEID);
  long (*on_color)(NODEID);
  long (*label_color)(NODEID);
  long (*mouse_color)(NODEID);

  char *(*summary)(NODEID);
  char *(*def_pick)(NODEID);
  char *(*auto_select)(NODEID);
  char *(*highlight)(NODEID);

  void (*on_entry)(NODEID);
  void (*on_exit)(NODEID);
  char *(*action)(NODEID);

  char *(*for_list)(NODEID);

  void (*pick)(NODEID);
  void (*no_pick)(NODEID);

  FILE *(*rfile)(NODEID);

  char *(*choice_set)(NODEID);
  char *(*choice_init)(NODEID);
  char *(*choice_allow)(NODEID);
  char *(*choice_disallow)(NODEID);
  long (*choice_min)(NODEID);
  long (*choice_max)(NODEID);

  int choice_key_case;

  long (*str_no_echo)(NODEID);

/* Several kinds needed...
   Say nothing at all to avoid type-checking conflicts! */

  char *(*term_encode)();

  long (*term_decode)();

  long (*term_width)(NODEID);

  long (*long_init)(NODEID);
  long (*long_min)(NODEID);
  long (*long_max)(NODEID);

  float (*float_init)(NODEID);
  float (*float_min)(NODEID);
  float (*float_max)(NODEID);

  double (*double_init)(NODEID);
  double (*double_min)(NODEID);
  double (*double_max)(NODEID);

#endif

} UINODE;

/* Access to parts of a node entry by index. */

#define Node_Name(nx) (Nodes[(nx)].name)
#define Node_Help(nx) (Nodes[(nx)].help)
#define Node_On_Cr(nx) (Nodes[(nx)].on_cr)

#define Node_Layout_Style(nx) (Nodes[(nx)].layout_style)

#define Node_Follows(nx) (Nodes[(nx)].follows)
#define Node_Overlays(nx) (Nodes[(nx)].overlays)

#define Node_Layout_Left(nx) (Nodes[(nx)].layout_left)
#define Node_Layout_Right(nx) (Nodes[(nx)].layout_right)
#define Node_Layout_Tab(nx) (Nodes[(nx)].layout_tab)
#define Node_Layout_VTab(nx) (Nodes[(nx)].layout_vtab)

#define Node_Place_X(nx) (Nodes[(nx)].place_x)
#define Node_Place_Y(nx) (Nodes[(nx)].place_y)

#define Node_Window_X(nx) (Nodes[(nx)].window_x)
#define Node_Window_Y(nx) (Nodes[(nx)].window_y)
#define Node_Window_W(nx) (Nodes[(nx)].window_w)
#define Node_Window_H(nx) (Nodes[(nx)].window_h)

#define Node_StrucSize(nx) (Nodes[(nx)].struc_size)
#define Node_Level(nx) (Nodes[(nx)].level)
#define Node_Kind(nx) (Nodes[(nx)].kind)
#define Node_Term_Kind(nx) (Nodes[(nx)].term_kind)

#define Node_Child(nx) (Nodes[(nx)].child)
#define Node_Sibling(nx) (Nodes[(nx)].sibling)
#define Node_Back(nx) (Nodes[(nx)].back)
#define Node_Popup(nx) (Nodes[(nx)].popup)

#define Node_Struct_Ptr(nx) (Nodes[(nx)].struct_ptr)

#define SOMEFLAG    1
#define ANOTHERONE  2

#define Node_Flags(nx) (Nodes[(nx)].flags)
#define Node_SetFlag(nx, flag, val) Nodes[(nx)].flags = (val) ? \
 (Nodes[(nx)].flags | flag) : (Nodes[(nx)].flags | flag) - flag;
#define Node_GetFlag(nx, flag) ( Nodes[(nx)].flags & flag )

#define Node_Spare(nx) (Nodes[(nx)].spare)

#define Node_Enable_Choice(nx) (Nodes[(nx)].enable_choice)
#define Node_Omit(nx) (Nodes[(nx)].omit)
#define Node_Click_Action(nx) (Nodes[(nx)].click_action)
#define Node_Auto_Ok(nx) (Nodes[(nx)].auto_ok)
#define Node_Read_Only(nx) (Nodes[(nx)].read_only)
#define Node_Enable_Ok(nx) (Nodes[(nx)].enable_ok)

#define Node_Remain(nx) (Nodes[(nx)].remain)
#define Node_Confirm(nx) (Nodes[(nx)].confirm)
#define Node_Conf_Msg(nx) (Nodes[(nx)].conf_msg)
#define Node_Discard(nx) (Nodes[(nx)].discard)
#define Node_Event_Test(nx) (Nodes[(nx)].event_test)
#define Node_No_Label(nx) (Nodes[(nx)].no_label)

#define Node_Fill_Color(nx) (Nodes[(nx)].fill_color)

#define Node_Body_Color(nx) (Nodes[(nx)].body_color)
#define Node_On_Color(nx) (Nodes[(nx)].on_color)
#define Node_Label_Color(nx) (Nodes[(nx)].label_color)
#define Node_Mouse_Color(nx) (Nodes[(nx)].mouse_color)

#define Node_Summary(nx) (Nodes[(nx)].summary)
#define Node_Def_Pick(nx) (Nodes[(nx)].def_pick)
#define Node_Auto_Select(nx) (Nodes[(nx)].auto_select)
#define Node_Highlight(nx) (Nodes[(nx)].highlight)

#define Node_On_Entry(nx) (Nodes[(nx)].on_entry)
#define Node_On_Exit(nx) (Nodes[(nx)].on_exit)
#define Node_Action(nx) (Nodes[(nx)].action)

#define Node_For_List(nx) (Nodes[(nx)].for_list)

#define Node_Pick(nx) (Nodes[(nx)].pick)
#define Node_No_Pick(nx) (Nodes[(nx)].no_pick)

#define Node_RFile(nx) (Nodes[(nx)].rfile)

#define Node_Choice_Set(nx) (Nodes[(nx)].choice_set)
#define Node_Choice_Init(nx) (Nodes[(nx)].choice_init)
#define Node_Choice_Allow(nx) (Nodes[(nx)].choice_allow)
#define Node_Choice_DisAllow(nx) (Nodes[(nx)].choice_disallow)
#define Node_Choice_Min(nx) (Nodes[(nx)].choice_min)
#define Node_Choice_Max(nx) (Nodes[(nx)].choice_max)

#define Node_Choice_Key_Case(nx) (Nodes[(nx)].choice_key_case)

#define Node_Str_No_Echo(nx) (Nodes[(nx)].str_no_echo)

#define Node_Term_Encode(nx) (Nodes[(nx)].term_encode)
#define Node_Term_Decode(nx) (Nodes[(nx)].term_decode)

#define Node_Term_Width(nx) (Nodes[(nx)].term_width)

#define Node_Long_Init(nx) (Nodes[(nx)].long_init)
#define Node_Long_Min(nx) (Nodes[(nx)].long_min)
#define Node_Long_Max(nx) (Nodes[(nx)].long_max)

#define Node_Float_Init(nx) (Nodes[(nx)].float_init)
#define Node_Float_Min(nx) (Nodes[(nx)].float_min)
#define Node_Float_Max(nx) (Nodes[(nx)].float_max)

#define Node_Double_Init(nx) (Nodes[(nx)].double_init)
#define Node_Double_Min(nx) (Nodes[(nx)].double_min)
#define Node_Double_Max(nx) (Nodes[(nx)].double_max)

/* dictionary supported data types. */

#define MALLOCVAL 0
#define LONGVAL   1
#define FLOATVAL  2
#define DOUBLEVAL 3
#define STRINGVAL 4
#define STRUCTVAL 5

/* data record */

typedef struct UIDATA {
  char *name;
  struct UIDATA *sibling;
  int kind;
  long seqstamp;
  long (*active)();
  long noappend;
  int struc_size;
 union {
   long   L;
   float  F[2];
   double D;
   char  *S;
 } value;
} UIDATA;


/* Access to parts of a data dictionary entry by pointer. */

#define Data_Name(dp) ((dp) -> name)
#define Data_Sibling(dp) ((dp) -> sibling)
#define Data_Kind(dp) ((dp) -> kind)
#define Data_SeqStamp(dp) ((dp) -> seqstamp)
#define Data_Active(dp) ((dp) -> active)
#define Data_NoAppend(dp) ((dp) -> noappend)

#define Data_StrucSize(dp) ((dp) -> struc_size)

#define Data_LValue(dp) ((dp) -> value.L)
#define Data_FValue(dp) ((dp) -> value.F[0])
#define Data_DValue(dp) ((dp) -> value.D)
#define Data_SValue(dp) ((dp) -> value.S)

#include "ui_heads.h"

#define NoAppend(name)  (Data_NoAppend(uiget(name)))

#define GetLong(name)   (Data_LValue((uiget(name)) ? \
 uiget(name) : uiinitval(name, INT_TERM)))
#define GetFloat(name)  (Data_FValue((uiget(name)) ? \
 uiget(name) : uiinitval(name, FLOAT_TERM)))
#define GetDouble(name) (Data_DValue((uiget(name)) ? \
 uiget(name) : uiinitval(name, DOUBLE_TERM)))
#define GetString(name) (Data_SValue((uiget(name)) ? \
 uiget(name) : uiinitval(name, STRING_TERM)))
#define GetMalloc(name) (Data_SValue(uiget(name)))
#define GetStruct(name) (Data_SValue(uiget(name)))

#define DataActive(name) (Data_Active(uiget(name)))
#define DataKind(name)	(Data_Kind(uiget(name)))
#define DataSeqStamp(name) (Data_SeqStamp(uiget(name)))

#define HaveValue(name) (uiget(name))

/* True when 1 is newer than 2 or 1 exists but 2 does not. */

#define Newer(name1,name2) \
 ((HaveValue(name1) && HaveValue(name2) \
&& (DataSeqStamp(name1) > DataSeqStamp(name2))) \
|| (HaveValue(name1) && !HaveValue(name2)))

/* Only when changed from current value... */

#define DefMalloc(name,val) \
 if (!HaveValue(name) || GetMalloc(name) != (val)) \
  uidef((name),MALLOCVAL,(val))

#define DefLong(name,val) \
 if (!HaveValue(name) || GetLong(name) != (val)) \
  uidef((name),LONGVAL,(val))

#define DefFloat(name,val) \
 if (!HaveValue(name) || GetFloat(name) != (val)) \
  uidef((name),FLOATVAL,(val))

#define DefDouble(name,val) \
 if (!HaveValue(name) || GetDouble(name) != (val)) \
  uidef((name),DOUBLEVAL,(val))

/* Always,
   because not obvious what should be tested.
   Test for changed pointer or contents or both?! */

#define DefString(name,val) \
  uidef((name),STRINGVAL,(val))

#define DefStruct(name,val) \
  uidef((name),STRUCTVAL,sizeof(val),&(val))

/* Always,
   but maybe these should have tests - at the level. */

#define StoreMalloc(node,name,val) uistore((node),(name),MALLOCVAL,(val))
#define StoreLong(node,name,val)   uistore((node),(name),LONGVAL,(val))
#define StoreFloat(node,name,val)  uistore((node),(name),FLOATVAL,(val))
#define StoreDouble(node,name,val) uistore((node),(name),DOUBLEVAL,(val))
#define StoreString(node,name,val) uistore((node),(name),STRINGVAL,(val))
#define StoreStruct(node,name,val) uistore((node),(name),STRUCTVAL,sizeof(val),&(val))

#define Cond(a,b) ((a) ? a(b) : 0)

#endif
