#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/StringDefs.h>
#include <X11/Xutil.h>
#include <X11/Xlib.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/AtomMgr.h>
#include <Xm/Protocols.h>

extern Display *cx_display;
extern Screen  *cx_screen;
extern Window   cx_rootwindow;

extern Widget   cx_toplevel;
extern Window   cx_topwindow;

static Atom    _MOTIF_WM_MENU = NULL;
static Atom    _MOTIF_WM_MESSAGES = NULL;
static Atom     WM_PROTOCOLS = NULL;

static int      pp;

static Atom   actual_type;
static int    actual_format;
static unsigned long nitems;
static unsigned long bytes_leftover;
static unsigned char *data = NULL;

static Atom  *proto_list;
static int    proto_count = 0, matched = 0;

/* Append below the last standard WM menu item,
   a SINGLE string holding any number of menu items,
   separated by newlines.

   The most useful formats are:

   "myItem _I Ctrl<Key>I f.send_msg N"

   where N is a known Atom value to be sent as a ClientMessage,
   and

   "my\\ Job f.exec NAME arg arg arg"

   where NAME is a program to execute.

   Both the _I mnemonic and Ctrl<Key>I accelerator parts are optional.
   Either or both if used must appear in the order shown.

   Embedded spaces in item names require a DOUBLE backslash.

   Note that for EVERY 'send_msg N' you must also register a Callback,
   which will be called with the message number as client_data,
   so that the CB routine can serve more than one message.

   cx_menu_CB(Atom message_atom, XtPointer messageCB)

   See 'man mwm' for more details. */

void cx_addmenu(char *menuadd)

{ static char   kws[] = "@(#)cx_addmenu.c	1.1 ARGOS 94/02/03 10:17:54";

/* c y a */

  if (!menuadd || !strlen(menuadd))
    return;

  if (!cx_display || !cx_topwindow) {
    printf("gui_init must be called before cx_addmenu\n");
    return;
  }

  if (!_MOTIF_WM_MENU)
    _MOTIF_WM_MENU =
      XmInternAtom(cx_display, "_MOTIF_WM_MENU", False);

  if (!_MOTIF_WM_MESSAGES)
    _MOTIF_WM_MESSAGES =
      XmInternAtom(cx_display, "_MOTIF_WM_MESSAGES", False);

  if (!WM_PROTOCOLS)
    WM_PROTOCOLS =
      XmInternAtom(cx_display, "WM_PROTOCOLS", False);

  XChangeProperty(cx_display, cx_topwindow,
          _MOTIF_WM_MENU,
          XA_STRING, 8,
          PropModeAppend,
          menuadd, strlen(menuadd) + 1);

/* Make sure that _MOTIF_WM_MESSAGES is one of our WM_PROTOCOLS.
   Should only append it if not yet present. */

  proto_count = 0;
  matched = 0;

  if ((XGetWindowProperty(cx_display, cx_topwindow,
              WM_PROTOCOLS,
              0, 1000, False, AnyPropertyType,
              &actual_type,
              &actual_format,
              &proto_count,
              &bytes_leftover,
              &data) == Success)
    && proto_count) {

    proto_list = (Atom *) data;

    for (pp = 0;
       !matched && (pp < proto_count);
       pp++)
      matched = (proto_list[pp] == _MOTIF_WM_MESSAGES);
  }

  if (!matched)
    XChangeProperty(cx_display, cx_topwindow,
            WM_PROTOCOLS,
            XA_ATOM, 32,
            PropModeAppend,
            _MOTIF_WM_MESSAGES, 1);
}

/* Make sure that message_atom is one of our _MOTIF_WM_MESSAGES.
   Should only append it if not yet present. */

void cx_menu_CB(Atom message_atom, XtPointer CB)
{

/* c y a */

  if (!message_atom || !CB)
    return;

  if (!cx_display || !cx_topwindow) {
    printf("gui_init must be called before cx_menu_CB\n");
    return;
  }

  if (!_MOTIF_WM_MESSAGES)
    _MOTIF_WM_MESSAGES =
      XmInternAtom(cx_display, "_MOTIF_WM_MESSAGES", False);

  proto_count = 0;
  matched = 0;

  if ((XGetWindowProperty(cx_display, cx_topwindow,
              _MOTIF_WM_MESSAGES,
              0, 1000, False, AnyPropertyType,
              &actual_type,
              &actual_format,
              &proto_count,
              &bytes_leftover,
              &data) == Success)
    && proto_count) {

    proto_list = (Atom *) data;

    for (pp = 0;
       !matched && (pp < proto_count);
       pp++)
      matched = (proto_list[pp] == message_atom);
  }

  if (!matched)
    XChangeProperty(cx_display, cx_topwindow,
            _MOTIF_WM_MESSAGES,
            XA_ATOM, 32,
            PropModeAppend,
            &message_atom, 1);

  XmAddProtocolCallback(cx_toplevel,
      _MOTIF_WM_MESSAGES, message_atom,
     (void *)CB, (void *)message_atom);
}
