#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/StringDefs.h>
#include <X11/Xutil.h>
#include <X11/Xlib.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/AtomMgr.h>
#include <Xm/Protocols.h>

extern Display *cx_display;
extern Screen  *cx_screen;
extern Window   cx_rootwindow;

extern Widget   cx_toplevel;
extern Window   cx_topwindow;

static Atom   WM_SAVE_YOURSELF = NULL;
static Atom   W_M_P = NULL;

/* Post a (revised) command line to our toplevel shell window
   as the pre-defined XA_WM_COMMAND property in XA_STRING form,
   containing one or more NULL-terminated words.

   A built-in call could be used but is slightly less flexible:

   XSetCommand(Display *display, Window topwindow, char **argv, int argc);

   Note the order 'argv, argc' reverse of the usual.

   Our version can start from a single argv[0] string with argc = 1,
   or an argc / argv pair with separate arguments
.
   If argc == 1 we attempt to break up argv[0] at spaces.
   Squeeze multiple spaces down to single because they may hurt.

   Note that the initial command line is posted for us;
   using this call allows the application to post updates.

   An app should update if it (interactively) changes its configuration,
   such that a new command would start it in this configuration,
   or it started with "extra information" that affected it,
   and may not be available the next time.

   It should also update in response to a WM_SAVE_YOURSELF message,
   if it has registered for and received such a message.
   The update in response to WM_SAVE_YOURSELF is required:
   Post a zero-length append if nothing new to say. */

void 
cx_postcmd(int argc, char *argv[])

{ static char   kws[] = "@(#)cx_postcmd.c	1.8 ARGOS 94/02/05 17:34:31";

  unsigned char  *wm_command = NULL;
  int       nn, valid, ii, bytes = 0;
  char      this;

  if (!cx_display || !cx_topwindow) {
    printf("gui_init must be called before cx_postcmd\n");
    return;
  }

  for (ii = 0; ii < argc; ii++)
    if (argv[ii])
      bytes += strlen(argv[ii]);

  if ((bytes > 0)
    && (wm_command = (unsigned char *) malloc(bytes + argc + 1))) {

    bytes = 0;
    for (ii = 0; ii < argc; ii++) {
      valid = 0;
      if (argv[ii])
        for (nn = 0; this = argv[ii][nn]; nn++) {
          if (this != ' ') {
            wm_command[bytes++] = this;
            valid = 1;
          }
          else if (valid) {
            wm_command[bytes++] = NULL;
            valid = 0;
          }
        }
      if (!valid)
        bytes--;
      wm_command[bytes++] = NULL;
    }

    XChangeProperty(cx_display, cx_topwindow,
       XA_WM_COMMAND, XA_STRING, 8, PropModeReplace, wm_command, bytes);

    free(wm_command);
  }

  else
    XChangeProperty(cx_display, cx_topwindow,
            XA_WM_COMMAND, XA_STRING, 8, PropModeAppend, "", 0);
}

/* Convenience to pre- (or ap-) pend an arg to the existing command. */

void 
cx_modcmd(int pre, char *pend)
{
  if (!cx_display || !cx_topwindow) {
    printf("gui_init must be called before cx_modcmd\n");
    return;
  }

  XChangeProperty(cx_display, cx_topwindow,
          XA_WM_COMMAND, XA_STRING, 8,
          (pre) ? PropModePrepend : PropModeAppend,
          pend, strlen(pend) + 1);
}
