/* The Active test is kind of hacked-in,
   should probably only apply to changes not new entries.
   The test is called with current struct and proposed value,
   kind of pointless if there is not yet a current value.

   Not yet doing this one for a couple of reasons:
   Only update an existing value and bump sequence counter
   if the value is actually changed.
   Be careful of zero vs new value.
   Not decided yet...
   Should this test come first and maybe skip the active call,
   or should the active call come first so it knows of attempt? */

/* Returns pointer to the dictionary node created or updated,
   using it with proper value member gives the def'd value. */

#include <varargs.h>
#include <string.h>
#include <stdlib.h>
#include <memory.h>
#include <malloc.h>

#include "gui_comm.h"

UIDATA * uidef(va_alist) va_dcl

/* Create or update an entry in current (top) dictionary.

   Entry is a descriptor with name and value or pointer.

   This uses the variable argument list macro package.
   The first argument is the name of the entry.
   The second is a code for data kind,
   which determines meaning of last. */

{ static char   vers[] = "@(#)uidef.c	1.1 ARGOS 93/12/13 17:37:13";

    int      *DIE = 0;

    long     ltemp;
    double   dtemp;

    char       *svp;

    char       *ename;
    int       kind;

    UIDATA     *ii;

    va_list     ap;

    va_start(ap);

    if (!(ename = va_arg(ap, char *))) {
        printf("uidef:zero for name\n");
        *DIE = 0;
    }
    else if (!strlen(ename)) {
        printf("uidef:null name\n");
        *DIE = 0;
    }
    else if (!Actives) {
        printf("uidef:empty Actives stack for def %s\n", ename);
        *DIE = 0;
    }

    kind = va_arg(ap, int);

/* Search only the current (top) dictionary,
   search terminates at null pointer. */

    for (ii = Data[Actives];
       ii;
       ii = Data_Sibling(ii)) {
        if (strcasecmp(ename, Data_Name(ii)) == 0)
            break;
    }

/* if found kinds requested and existing must match. */

    if (ii) {
        if (Data_Kind(ii) != kind) {
            printf("uidef error:%s kinds requests %d vs current %d\n",
                   ename, kind, Data_Kind(ii));
            *DIE = 0;
        }
    }

/* if not found add a new entry. */

    else {
        ii = (UIDATA *) malloc(sizeof(UIDATA));
	memset(ii, 0x00, sizeof(UIDATA));
        Data_Sibling(ii) = Data[Actives];
        Data[Actives] = ii;

        Data_Name(ii) = strdup(ename);
        Data_Kind(ii) = kind;
        Data_SValue(ii) = 0;
    }

/* Sequence stamp all data creation and updates. */

    Data_SeqStamp(ii) = ++SeqStamp;

    switch (kind) {

    case LONGVAL:
        ltemp = va_arg(ap, long);
	if (Data_Active(ii) && Data_Active(ii)(ii, ltemp) == 0)
	    return ii;

        Data_LValue(ii) = ltemp;
#ifdef DEBUG
        if (DEBUG)
            fprintf(DEBUG,
             "def LONG %ld %s,%s as %d\n",
             Data_SeqStamp(ii), Node_Name(UITOP), Data_Name(ii), Data_LValue(ii));
#endif
        break;

/* Passed to us as double not float in spite of how we declare it,
   (tried float variable and (float) cast on variable or literal)
   so take it here as a double and then stuff into float. */

    case FLOATVAL:
        dtemp = va_arg(ap, float);
	if (Data_Active(ii) && Data_Active(ii)(ii, dtemp) == 0)
	    return ii;

        Data_FValue(ii) = dtemp;
#ifdef DEBUG
        if (DEBUG)
            fprintf(DEBUG,
             "def FLOAT %ld %s,%s as %f\n",
             Data_SeqStamp(ii), Node_Name(UITOP), Data_Name(ii), Data_FValue(ii));
#endif
        break;

    case DOUBLEVAL:
        dtemp = va_arg(ap, double);
	if (Data_Active(ii) && Data_Active(ii)(ii, dtemp) == 0)
	    return ii;

        Data_DValue(ii) = dtemp;
#ifdef DEBUG
        if (DEBUG)
            fprintf(DEBUG,
             "def DOUBLE %ld %s,%s as %lf\n",
             Data_SeqStamp(ii), Node_Name(UITOP), Data_Name(ii), Data_DValue(ii));
#endif
        break;

    case STRINGVAL:
        svp = (va_arg(ap, char *));
	if (Data_Active(ii) && Data_Active(ii)(ii, svp) == 0)
	    return ii;

        if (Data_SValue(ii) != svp) {
            if (Data_SValue(ii))
                free(Data_SValue(ii));
            Data_SValue(ii) = strdup(svp);
        }
#ifdef DEBUG
        if (DEBUG)
            fprintf(DEBUG,
             "def STRING %ld %s,%s as %s\n",
             Data_SeqStamp(ii), Node_Name(UITOP), Data_Name(ii), Data_SValue(ii));
#endif
        break;

/* pass the size of other (structured) data. */

    case STRUCTVAL:
        ltemp = va_arg(ap, long);
        svp = (va_arg(ap, char *));

	if (Data_Active(ii) && Data_Active(ii)(ii, svp) == 0)
	    return ii;
        
        if (Data_SValue(ii) != svp) {
            if (Data_SValue(ii))
                free(Data_SValue(ii));
            Data_SValue(ii) = memcpy((void *)malloc(ltemp),
                                     (void *)svp,
                                     (size_t)ltemp);
        }
#ifdef DEBUG
        if (DEBUG)
            fprintf(DEBUG,
             "def STRUCT %ld %s,%s size: %ld\n",
             Data_SeqStamp(ii), Node_Name(UITOP), Data_Name(ii), ltemp);
#endif
        break;

/* just store a malloc pointer provided by the caller.
   release any previous allocation. */

    case MALLOCVAL:
        svp = (va_arg(ap, char *));

	if (Data_Active(ii) && Data_Active(ii)(ii, svp) == 0)
	    return ii;

        if (Data_SValue(ii) != svp) {
            if (Data_SValue(ii))
                free(Data_SValue(ii));
            Data_SValue(ii) = svp;
        }
#ifdef DEBUG
        if (DEBUG)
            fprintf(DEBUG,
             "def MALLOC %ld %s,%s as %ld\n",
             Data_SeqStamp(ii), Node_Name(UITOP), Data_Name(ii), Data_SValue(ii));
#endif
        break;
    }

    va_end(ap);

#ifdef DEBUG
    if (DEBUG) fflush(DEBUG);
#endif
}
